from asgi_aiogram.aliases import Receiver

async def read_body(receive: Receiver) -> bytes:
    body = b''
    more_body = True

    while more_body:
        message = await receive()
        body += message.get('body', b'')
        more_body = message.get('more_body', False)

    return body